/*
 * Simple Graphical User Interface with XView
 *
 * TeXchan version 2.1 for compiling TeX and so on
 *
 * copyleft 1992-1994 Taiji YAMADA and Natsuhiro ICHINOSE
 */
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/tty.h>
#include <xview/termsw.h>
#include <xview/textsw.h>
#include <xview/scrollbar.h>
#include <xview/notice.h>
#include <xview/cms.h>
#include "texchan.h"
#include "files.h"

#define LEN 256

char target[LEN] = "";
char texfilename[LEN], texcommand[LEN], dviselect[LEN], dviviewer[LEN];
char psviewer[LEN], pssize[LEN], psfilename[LEN];
enum TeXchanModeNo{
  LatexMode,
  DviViewMode,
  DviPrintMode,
  Dvi2PsMode,
  PsViewMode,
  PsPrintMode,
};
int TeXchanMode = LatexMode;

void lstarget_proc();
void lspsname_proc();
Xv_object set_target_proc();
Xv_object set_psname_proc();

int getsettings();

main(argc,argv)
int argc;
char *argv[];
{

  xv_init(XV_INIT_ARGC_PTR_ARGV,&argc,argv,
#ifdef JLE
	  XV_USE_LOCALE, TRUE,
	  /******XV_LC_DISPLAY_LANG, "japanese",
	    XV_LC_INPUT_LANG, "japanese",******/
#endif
	  NULL);

#include "texchan.code"

  if ( argc > 1  ){
    xv_set( text_target, PANEL_VALUE, argv[1], NULL );
    sprintf( target, "%s", (char *)xv_get( text_target, PANEL_VALUE ) );
  }
  getsettings();

  xv_set( text_target, PANEL_EVENT_PROC, lstarget_proc, 0 );
  xv_set( text_psname, PANEL_EVENT_PROC, lspsname_proc, 0 );

  xv_main_loop(BASE_FRAME);
  exit(0);
}

int button_latex_proc(item,event)
Panel_item item;
Event *event;
{
  char str[LEN];

  TeXchanMode = LatexMode;
  if ( !getsettings() ) return 0;
  xv_set( button_1, PANEL_LABEL_STRING, "X", 0 );
  xv_set( button_2, PANEL_LABEL_STRING, "H", 0 );
  sprintf( str, "%s %s.tex\n", texcommand, target );
  ttysw_input( ttywindow, str, strlen(str) );

  auto_connect(item,event);
  return 1;
}

int button_dviview_proc(item,event)
Panel_item item;
Event *event;
{
  char str[LEN];
  
  auto_connect(item,event);

  TeXchanMode = DviViewMode;
  if ( !getsettings() ) return 0;
  xv_set( button_1, PANEL_LABEL_STRING, "Q", 0 );
  xv_set( button_2, PANEL_LABEL_STRING, ">", 0 );
  sprintf( str, "%s %s.dvi\n", dviviewer, target );
  ttysw_input( ttywindow, str, strlen(str) );
  return 1;
}

int button_dviprint_proc(item,event)
Panel_item item;
Event *event;
{
  char str[LEN];
  
  auto_connect(item,event);

  TeXchanMode = DviPrintMode;
  if ( !getsettings() ) return 0;
  /*xv_set( button_1, PANEL_LABEL_STRING, "X", 0 );
  xv_set( button_2, PANEL_LABEL_STRING, "H", 0 );*/
  sprintf( str, "lpr -d -Ptexlbp %s.dvi\n", target );
  ttysw_input( ttywindow, str, strlen(str) );
  return 1;
}

int button_dvi2ps_proc(item,event)
Panel_item item;
Event *event;
{
  char str[LEN];
  
  auto_connect(item,event);

  TeXchanMode = Dvi2PsMode;
  if ( !getsettings() ) return 0;
  /*xv_set( button_1, PANEL_LABEL_STRING, "X", 0 );
  xv_set( button_2, PANEL_LABEL_STRING, "H", 0 );*/
  sprintf( str, "\\rm %s.ps\n", psfilename );
  ttysw_input( ttywindow, str, strlen(str) );
  sprintf( str, "dvi2ps -o %s %s.dvi > %s.ps\n", pssize, target, psfilename );
  ttysw_input( ttywindow, str, strlen(str) );
  return 1;
}

int button_psview_proc(item,event)
Panel_item item;
Event *event;
{
  char str[LEN];
  
  auto_connect(item,event);

  TeXchanMode = PsViewMode;
  if ( !getsettings() ) return 0;
  xv_set( button_1, PANEL_LABEL_STRING, "Q", 0 );
  xv_set( button_2, PANEL_LABEL_STRING, ">", 0 );
  sprintf( str, "%s %s.ps\n", psviewer, psfilename );
  ttysw_input( ttywindow, str, strlen(str) );
}

int button_psprint_proc(item,event)
Panel_item item;
Event *event;
{
  char str[LEN];
  
  auto_connect(item,event);

  TeXchanMode = PsPrintMode;
  if ( !getsettings() ) return 0;
  /*xv_set( button_1, PANEL_LABEL_STRING, "X", 0 );
  xv_set( button_2, PANEL_LABEL_STRING, "H", 0 );*/
  sprintf( str, "pslpr %s.ps\n", psfilename );
  ttysw_input( ttywindow, str, strlen(str) );
  return 1;
}

int button_quit_proc(item,event)
Panel_item item;
Event *event;
{
  auto_connect(item,event);
  exit(0);
}

int button_1_proc(item,event)
Panel_item item;
Event *event;
{
  auto_connect(item,event);

  switch( TeXchanMode ){
  case LatexMode:
    ttysw_input( ttywindow, "x\n", 2 );
    break;
  case PsViewMode:
    ttysw_input( ttywindow, "quit\n", 5 );
    break;
  }
  return 1;
}

int button_2_proc(item,event)
Panel_item item;
Event *event;
{
  auto_connect(item,event);

  switch( TeXchanMode ){
  case LatexMode:
    ttysw_input( ttywindow, "h\n", 2 );
    break;
  case PsViewMode:
    ttysw_input( ttywindow, "\n", 1 );
    break;
  }
  return 1;
}

int button_dviselect_proc(item,event)
Panel_item item;
Event *event;
{
  char str[LEN];
  
  auto_connect(item,event);

  if ( !getsettings() ) return 0;
  /*xv_set( button_1, PANEL_LABEL_STRING, "X", 0 );
  xv_set( button_2, PANEL_LABEL_STRING, "H", 0 );
  TeXchanMode = LatexMode;*/
  if ( strlen(dviselect)!=0 ){
    sprintf( str, "\\rm %s_.dvi\n ",target );	
    ttysw_input( ttywindow, str, strlen(str) ); 	
    sprintf( str, "dviselect %s.dvi =%s > %s_.dvi\n",
	    target, dviselect, target );	
    ttysw_input( ttywindow, str, strlen(str) );
    sprintf( str, "\\rm %s.dvi\n ", target );	
    ttysw_input( ttywindow, str, strlen(str) );
    sprintf( str, "mv %s_.dvi %s.dvi\n ", target, target );	
    ttysw_input( ttywindow, str, strlen(str) );
  }
  return 1;
}

Xv_object set_target_proc( menu, item )
Menu menu;
Menu_item item;
{
  char *ptr, str[LEN];

  auto_connect(item,NULL);

  strcpy( str, (char*)xv_get( item, MENU_STRING ) );
  ptr = strstr( str, ".tex" );
  if ( ptr != NULL ){
    *ptr = '\0';
    xv_set( text_target, PANEL_VALUE, str, NULL );
  }
  getsettings();
  return( item );
}

void lstarget_proc(panel,event)
Panel panel;
Event *event;
{
  int i, n;
  Menu menu;
  Menu_item mi;
  char **files;

  auto_connect(panel,event);
  panel_default_handle_event(panel,event);

  if ( event_action(event)!=ACTION_MENU || !event_is_down(event) ){
    return;
  }
  files = files_suffix( "./", "tex", &n );
  if ( n == 0 ){
    auto_connect(panel,event);
    panel_default_handle_event(panel,event);
    return;
  }
  menu = (Menu)xv_create( NULL, MENU,
			 MENU_TITLE_ITEM, "tex files",
			 MENU_NOTIFY_PROC, set_target_proc,
			 NULL);
  for ( i=0; i<n; i++ ){
    mi = (Menu_item)xv_create( NULL, MENUITEM,
			      MENU_STRING, files[i],
			      MENU_NOTIFY_PROC, set_target_proc,
			      MENU_RELEASE,
			      NULL);
    xv_set( menu, MENU_APPEND_ITEM, mi, NULL );
  }
  menu_show( menu, panel_main, event, NULL );
}

Xv_object set_psname_proc( menu, item )
Menu menu;
Menu_item item;
{
  char *ptr, str[LEN];

  auto_connect(item,NULL);

  strcpy( str, (char*)xv_get( item, MENU_STRING ) );
  ptr = strstr( str, ".ps" );
  if ( ptr != NULL ){
    *ptr = '\0';
    xv_set( text_psname, PANEL_VALUE, str, NULL );
  }
  getsettings();
  return( item );
}

void lspsname_proc(panel,event)
Panel panel;
Event *event;
{
  int i, n;
  Menu menu;
  Menu_item mi;
  char **files;
  static char str[LEN];

  auto_connect(panel,event);
  panel_default_handle_event(panel,event);
  
  if ( event_action(event)!=ACTION_MENU || !event_is_down(event) ){
    return;
  }
  files = files_suffix( "./", "ps", &n );
  menu = (Menu)xv_create( NULL, MENU,
			 MENU_TITLE_ITEM, "ps files",
			 MENU_NOTIFY_PROC, set_psname_proc,
			 NULL);
  /* 1st def ps file */
  mi = (Menu_item)xv_create( NULL, MENUITEM,
			    MENU_STRING, "texchan.ps",
			    MENU_NOTIFY_PROC, set_psname_proc,
			    MENU_RELEASE,
			    NULL);
  xv_set( menu, MENU_APPEND_ITEM, mi, NULL );
  /* 2nd def ps file */
  if ( strncmp(target,"",LEN)!=0 ){
    strcpy( str, target );
    strcat( str, ".ps" );
    mi = (Menu_item)xv_create( NULL, MENUITEM,
			      MENU_STRING, str,
			      MENU_NOTIFY_PROC, set_psname_proc,
			      MENU_RELEASE,
			      NULL);
    xv_set( menu, MENU_APPEND_ITEM, mi, NULL );
  }
  for ( i=0; i<n; i++ ){
    if ( strncmp(files[i],"texchan.ps",LEN)!=0 &&
	strncmp(files[i],str,LEN)!=0 ){
      mi = (Menu_item)xv_create( NULL, MENUITEM,
				MENU_STRING, files[i],
				MENU_NOTIFY_PROC, set_psname_proc,
				MENU_RELEASE,
				NULL);
      xv_set( menu, MENU_APPEND_ITEM, mi, NULL );
    }
  }
  menu_show( menu, panel_main, event, NULL );
}

int getsettings()
{
  FILE *fp;
  char str[LEN];
  
  strcpy( texcommand,
	 (char*)xv_get( choice_texcommand, PANEL_CHOICE_STRING,
		xv_get(choice_texcommand,  PANEL_VALUE) ) );
  strcpy( dviviewer,
	 (char*)xv_get( choice_dviviewer, PANEL_CHOICE_STRING,
		xv_get(choice_dviviewer,  PANEL_VALUE) ) );
  strcpy( pssize,
	 (char*)xv_get( choice_pssize,    PANEL_CHOICE_STRING,
		xv_get(choice_pssize,     PANEL_VALUE) ) );
  strcpy( psviewer,
	 (char*)xv_get( choice_psviewer,  PANEL_CHOICE_STRING,
		xv_get(choice_psviewer,   PANEL_VALUE) ) );
  strcpy( target,     (char*)xv_get( text_target,    PANEL_VALUE, 0 ) );
  strcpy( psfilename, (char*)xv_get( text_psname,    PANEL_VALUE, 0 ) );
  strcpy( dviselect,  (char*)xv_get( text_dviselect, PANEL_VALUE, 0 ) );
  strcpy( texfilename, target );
  strcat( texfilename, ".tex" );

  if ( strstr( dviviewer, "xdvi" ) == dviviewer ){
    if (strcmp(pssize,"a4")==0) strcpy(dviviewer,"xdvi -paper a4");
    if (strcmp(pssize,"a4landscape")==0) strcpy(dviviewer,"xdvi -paper a4r");
    if (strcmp(pssize,"a3")==0) strcpy(dviviewer,"xdvi -paper a3");
    if (strcmp(pssize,"a3landscape")==0) strcpy(dviviewer,"xdvi -paper a3r");
    if (strcmp(pssize,"a5")==0) strcpy(dviviewer,"xdvi -paper a5");
    if (strcmp(pssize,"a5landscape")==0) strcpy(dviviewer,"xdvi -paper a5r");
    if (strcmp(pssize,"b3")==0) strcpy(dviviewer,"xdvi -paper b3");
    if (strcmp(pssize,"b3landscape")==0) strcpy(dviviewer,"xdvi -paper b3r");
    if (strcmp(pssize,"b4")==0) strcpy(dviviewer,"xdvi -paper b4");
    if (strcmp(pssize,"b4landscape")==0) strcpy(dviviewer,"xdvi -paper b4r");
    if (strcmp(pssize,"b5")==0) strcpy(dviviewer,"xdvi -paper b5");
    if (strcmp(pssize,"b5landscape")==0) strcpy(dviviewer,"xdvi -paper b5r");
  }

  switch( TeXchanMode ){
  case LatexMode:
  case DviViewMode:
  case DviPrintMode:
  case Dvi2PsMode:
  case PsViewMode:
  case PsPrintMode:
    if ( strlen(target)!=0 )
      if ( (fp=fopen( texfilename, "r" )) == NULL ){
	sprintf( str, "%s is not found !\n", texfilename );
	ttysw_output( ttywindow, str, strlen(str) );
	return( 0 );
      }else
	close( fp );
    else{
      sprintf( str, "Target name is unknown.\n" );
      ttysw_output( ttywindow, str, strlen(str) );
      return( 0 );
    }
    break;
  }
  return( 1 );
}

int getsettings_proc(item,event)
Panel_item item;
Event *event;
{
  getsettings();

  auto_connect(item,event);
  return 1;
}

int change_texcommand_proc(item,event)
Panel_item item;
Event *event;
{
  auto_connect(item,event);
  strcpy( texcommand,
	 (char*)xv_get( choice_texcommand, PANEL_CHOICE_STRING,
		xv_get(choice_texcommand,  PANEL_VALUE) ) );
  xv_set( button_latex, PANEL_LABEL_STRING, texcommand, 0 );
  return 1;
}
